/*
 * Minimize.java by Richard J. Davies
 * from `Introductory Java for Scientists and Engineers'
 * chapter: `Advanced Java'
 * section: `Example - Minimizing over a Region'
 *
 * This program finds the minimum of a function over a region. It assumes that
 * the region is within |x| < 1, |y| < 1, and places a square grid across this
 * large area. It then steps across the grid, recording the values of the
 * function whenever a grid point is within the region. This program is
 * intended to demonstrate use of multiple .java files, packages and import.
 */
package minimize.partone;

// Import minimize.parttwo so we can use Region
import minimize.parttwo.*;

public class Minimize
{
  // Define the fineness of the grid with
  // which we step across the region
  public static final int STEPS = 500;


  // The main method.
  public static void main(String[] argv)
  {
    double lowest, x, y;

    // Set lowest to an unachievably high value
    lowest = Double.POSITIVE_INFINITY;

    // and then step across the grid, updating
    // lowest every time we find a lower point
    // which is inside the region.
    for (x = -1 ; x <= 1 ; x += 1.0/STEPS)
      for (y = -1; y <= 1; y += 1.0/STEPS)
      {
        if ((Function.evaluate(x,y) < lowest)
            && Region.isInside(x,y))
        {
          lowest = Function.evaluate(x,y);
        }
      }

    System.out.println(lowest);
  }
}
