/*
 * Plottable.java, Copyright Richard J. Davies, 1998
 * from `Introductory Java for Scientists and Engineers'
 * chapter: `JSGL, a Scientific Graphics Library for Java'
 *
 * This is part of the implementation of the library.
 * 
 * The JSGL is Copyright (C) 1997-1999 Richard Davies.
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of either:
 * 
 *    i) the GNU Library General Public License; either version 2, or (at your
 *       option) any later version, or
 *
 *   ii) the "Artistic License".
 *
 * Copies of both licenses are enclosed with the library.
 */
package uk.co.jscieng;


/**
 * The Plottable interface is used to pass methods as arguments to the
 * <CODE>functionPlot</CODE> commands.
 *
 * <P>To pass a function to be graphed as an argument to <CODE>functionPlot
 * </CODE>, we define a class to contain the function which implements the
 * <CODE>Plottable</CODE> interface and defines the method <CODE>f</CODE>. This
 * will look something like:
 *
 * <PRE>
 * public class FunctionClass extends Plottable
 * {
 *   public double f(double x)
 *   {
 *     ...
 *   }
 * }
 * </PRE>
 *
 * We can then pass this method as an argument to <CODE>functionPlot</CODE> by
 * creating an object of this class and passing that object:
 *
 * <PRE>
 * FunctionClass o = new FunctionClass();
 * SciGraph.functionPlot(o);
 * </PRE>
 *
 * @author Richard J. Davies
 * @version 1.0
 * @see SciGraph#functionPlot
 */
public interface Plottable
{
  /**
   * The method <CODE>f</CODE> should be overridden in your class to calculate
   * whatever function you wish to graph.
   *
   * @param x The x-coordinate at which the function is to be evaluated.
   * @return The y-coordinate at that point.
   * @see Plottable
   */
  public double f(double x);
}
