/*
 * ODE.java by Richard J. Davies
 * from `Introductory Java for Scientists and Engineers'
 * chapter: `JSGL, a Scientific Graphics Library for Java'
 * section: `Example - ODE Contours'
 *
 * We plot a phase portrait for the differential equation governing a simple
 * pendulum.
 */
import uk.co.jscieng.*;

class ODE
{
  // Constants defining an area of phase plane.
  public static final double LEFT = -1;
  public static final double RIGHT = 7;
  public static final double BOTTOM = -2;
  public static final double TOP = 2;

  // The number of points across either side to
  // use for the lattice.
  public static final int POINTS = 30;


  // The main method
  public static void main(String[] argv)
  {
    double x, y, dx, dy;

    // Start the JSGL
    SciGraph.showGraph(LEFT, RIGHT, BOTTOM, TOP);

    // Step through all points in the lattice.
    for (int i=0; i<=POINTS; i++)
      for (int j=0; j<=POINTS; j++)
      {
        // Calculating their real coordinates.
        x = LEFT + i * (RIGHT - LEFT) / POINTS;
        y = BOTTOM + j * (TOP - BOTTOM) / POINTS;

        // Work out the derivative at that point.
        dx = y;
        dy = - Math.sin(x);

        // And draw a small flow line from the
        // lattice point in that direction.
        SciGraph.line(x, y, x + dx/10, y + dy/10);
      }
  }
}
